@@ -11,7 +11,7 @@ from pathlib import Path
 import numpy as np
 import plotly.graph_objects as go
 from openpyxl import Workbook
-from scipy.interpolate import interp1d
+from scipy.interpolate import interp1d, UnivariateSpline
 from scipy.optimize import fsolve
 
 from ccp import Q_, State, Point, Curve
@@ -184,6 +184,8 @@ class Impeller:
             "head",
             "eff",
             "power",
+            "psi",
+            "phi",
         ]:
             values = []
             # for disch.p etc values are defined in _Impeller_State
@@ -208,88 +210,6 @@ class Impeller:
             else:
                 return points_other == points_self
 
-    # def plot_func(self, attr):
-    #     @check_units
-    #     def inner(flow_v=None, speed=None, plot_kws=None, **kwargs):
-    #         """Plot parameter versus volumetric flow.
-    #
-    #         You can plot an specific point in the plot giving its flow_v and speed.
-    #
-    #         You can choose units with the arguments flow_v_units='...' and
-    #         {attr}_units='...'. For the speed you can use speed_units='...'.
-    #
-    #         Parameters
-    #         ----------
-    #         flow_v : pint.Quantity, float, optional
-    #             Volumetric flow (m³/s) for a specific point in the plot.
-    #         speed : pint.Quantity, float, optional
-    #             Speed (rad/s) for a specific point in the plot.
-    #         flow_v_units : str, optional
-    #             Flow units used for the plot. Default is m³/s.
-    #         {attr}_units : str, optional
-    #             Units for the parameter being plotted (e.g. for a head plot we could use
-    #             head_units='J/kg' or head_units='J/g'. Default is SI.
-    #         speed_units : str, optional
-    #             Speed units for the plot. Default is 'rad/s'.
-    #
-    #         Returns
-    #         -------
-    #         fig : plotly.Figure
-    #             Plotly figure that can be customized.
-    #
-    #         Examples
-    #         --------
-    #         >>> import ccp
-    #         >>> imp = ccp.impeller_example()
-    #         >>> fig = imp.plot_head(
-    #         ...    flow_v=5.5,
-    #         ...    speed=900,
-    #         ...    flow_v_units='m³/h',
-    #         ...    head_units='j/kg',
-    #         ...    speed_units='RPM'
-    #         ... )
-    #
-    #         """
-    #         fig = kwargs.pop("fig", None)
-    #
-    #         if fig is None:
-    #             fig = go.Figure()
-    #
-    #         if plot_kws is None:
-    #             plot_kws = {}
-    #
-    #         p0 = self.points[0]
-    #         flow_v_units = kwargs.get("flow_v_units", p0.flow_v.units)
-    #
-    #         for curve in self.curves:
-    #             fig = r_getattr(curve, attr + "_plot")(
-    #                 fig=fig, plot_kws=plot_kws, **kwargs
-    #             )
-    #
-    #         if speed:
-    #             current_curve = self.curve(speed=speed)
-    #             fig = r_getattr(current_curve, attr + "_plot")(
-    #                 fig=fig, plot_kws=plot_kws, **kwargs
-    #             )
-    #             if flow_v:
-    #                 current_point = self.point(flow_v=flow_v, speed=speed)
-    #                 fig = r_getattr(current_point, attr + "_plot")(
-    #                     fig=fig, plot_kws=plot_kws, **kwargs
-    #                 )
-    #
-    #         # extra x range
-    #         flow_values = [p.flow_v.to(flow_v_units) for p in self.points]
-    #         min_flow = min(flow_values)
-    #         max_flow = max(flow_values)
-    #         delta = 0.05 * (max_flow - min_flow)
-    #         fig.update_layout(xaxis=dict(range=[min_flow - delta, max_flow + delta]))
-    #
-    #         return fig
-    #
-    #     inner.__doc__ = r_getattr(self.curves[0], attr + "_plot").__doc__
-    #
-    #     return inner
-
     @check_units
     def point(self, flow_v=None, flow_m=None, speed=None):
         """Calculate specific point in the performance map.
@@ -410,7 +330,7 @@ class Impeller:
         return current_curve
 
     @classmethod
-    def convert_from(cls, original_impeller, suc=None, find="speed"):
+    def convert_from(cls, original_impeller, suc=None, find="speed", speed=None):
         """Convert performance map from an impeller.
 
         Parameters
@@ -424,6 +344,8 @@ class Impeller:
             For now only 'speed' is implemented, which means that, based on volume ratio,
             we calculate new values of speed for each curve and the respective discharge
             condition.
+        speed : float, pint.Quantity, optional
+            Desired speed. If find="speed", this should be None.
 
         Returns
         -------
@@ -437,14 +359,15 @@ class Impeller:
                 converter_args = [(p, suc, find) for p in curve]
                 converted_points = pool.map(converter, converter_args)
 
-                speed_mean = np.mean([p.speed.magnitude for p in converted_points])
+                if speed is None:
+                    speed = np.mean([p.speed.magnitude for p in converted_points])
 
                 converted_points = [
                     Point.convert_from(
                         p,
                         suc=p.suc,
                         find="volume_ratio",
-                        speed=speed_mean,
+                        speed=speed,
                     )
                     for p in converted_points
                 ]
@@ -508,14 +431,19 @@ class Impeller:
     def load_from_dict(
         cls,
         suc,
-        head_curves,
-        eff_curves,
+        head_curves=None,
+        eff_curves=None,
+        power_curves=None,
         b=None,
         D=None,
         number_of_points=10,
         flow_units="m**3/s",
         head_units="J/kg",
+        power_units="W",
         speed_units="RPM",
+        head_interpolation_method="interp1d",
+        eff_interpolation_method="interp1d",
+        power_interpolation_method="interp1d",
     ):
         """Create points from dict object.
 
@@ -548,6 +476,15 @@ class Impeller:
             If the curve head units are in meter you can use: head_units="m*g0".
         speed_units : str
             Speed units used in the dict.
+        head_interpolation_method : str, optional
+            Interpolation method from scipy. Can be interp1d or UnivariateSpline.
+            Default is interp1d.
+        eff_interpolation_method : str, optional
+            Interpolation method from scipy. Can be interp1d or UnivariateSpline.
+            Default is interp1d.
+        power_interpolation_method : str, optional
+            Interpolation method from scipy. Can be interp1d or UnivariateSpline.
+            Default is interp1d.
         """
         # define if we have volume or mass flow
         flow_type = "volumetric"
@@ -563,43 +500,127 @@ class Impeller:
                 kind=3,
                 fill_value="extrapolate",
             )
-            eff_curve = eff_curves[speed]
-            # check eff scale
-            if max(eff_curve["y"]) > 1:
-                eff_curve["y"] = [i / 100 for i in eff_curve["y"]]
-            eff_interpolated = interp1d(
-                eff_curve["x"],
-                eff_curve["y"],
-                kind=3,
-                fill_value="extrapolate",
+            head_spline = UnivariateSpline(
+                head_curve["x"],
+                head_curve["y"],
+            )
+            if eff_curves:
+                eff_curve = eff_curves[speed]
+                # check eff scale
+                if max(eff_curve["y"]) > 1:
+                    eff_curve["y"] = [i / 100 for i in eff_curve["y"]]
+                eff_interpolated = interp1d(
+                    eff_curve["x"],
+                    eff_curve["y"],
+                    kind=3,
+                    fill_value="extrapolate",
+                )
+                eff_spline = UnivariateSpline(
+                    eff_curve["x"],
+                    eff_curve["y"],
+                )
+            if power_curves:
+                power_curve = power_curves[speed]
+                power_interpolated = interp1d(
+                    power_curve["x"],
+                    power_curve["y"],
+                    kind=3,
+                    fill_value="extrapolate",
+                )
+                power_spline = UnivariateSpline(
+                    power_curve["x"],
+                    power_curve["y"],
+                )
+
+            # check for possible interpolation error by comparing interp1d and spline
+            # avoid too much extrapolation
+            if eff_curves:
+                min_x = max(head_curve["x"][0], eff_curve["x"][0])
+                max_x = min(head_curve["x"][-1], eff_curve["x"][-1])
+                points_x = np.linspace(min_x, max_x, number_of_points)
+            if power_curves:
+                min_x = max(head_curve["x"][0], power_curve["x"][0])
+                max_x = min(head_curve["x"][-1], power_curve["x"][-1])
+                points_x = np.linspace(min_x, max_x, number_of_points)
+
+            head_max_error = max(
+                1 - (head_interpolated(points_x) / head_spline(points_x))
             )
 
-            # avoid too much extrapolation
-            min_x = max(head_curve["x"][0], eff_curve["x"][0])
-            max_x = min(head_curve["x"][-1], eff_curve["x"][-1])
-
-            points_x = np.linspace(min_x, max_x, number_of_points)
-            points_head = head_interpolated(points_x)
-            points_eff = eff_interpolated(points_x)
-
-            args_list = [
-                (
-                    suc,
-                    Q_(float(speed), speed_units),
-                    Q_(flow, flow_units),
-                    Q_(head, head_units),
-                    eff,
-                    b,
-                    D,
+            if eff_curves:
+                eff_max_error = max(
+                    1 - (eff_interpolated(points_x) / eff_spline(points_x))
+                )
+            if power_curves:
+                power_max_error = max(
+                    1 - (power_interpolated(points_x) / power_spline(points_x))
                 )
-                for flow, head, eff in zip(points_x, points_head, points_eff)
-            ]
 
-            with multiprocessing.Pool() as pool:
-                if flow_type == "volumetric":
-                    points += pool.map(create_points_flow_v, args_list)
-                else:
-                    points += pool.map(create_points_flow_m, args_list)
+            if head_max_error > 0.1:
+                warnings.warn(
+                    f"Head interpolation error in speed {speed} {speed_units}.\n"
+                )
+            if eff_curves:
+                if eff_max_error > 0.1:
+                    warnings.warn(
+                        f"Efficiency interpolation error in speed {speed} {speed_units}.\n"
+                    )
+            if power_curves:
+                if power_max_error > 0.1:
+                    warnings.warn(
+                        f"Power interpolation error in speed {speed} {speed_units}.\n"
+                    )
+
+            if head_interpolation_method == "UnivariateSpline":
+                head_interpolated = head_spline
+            if eff_curves:
+                if eff_interpolation_method == "UnivariateSpline":
+                    eff_interpolated = eff_spline
+            if power_curves:
+                if power_interpolation_method == "UnivariateSpline":
+                    power_interpolated = power_spline
+
+            points_head = head_interpolated(points_x)
+
+            if eff_curves:
+                points_eff = eff_interpolated(points_x)
+                args_list = [
+                    (
+                        suc,
+                        Q_(float(speed), speed_units),
+                        Q_(flow, flow_units),
+                        Q_(head, head_units),
+                        eff,
+                        b,
+                        D,
+                    )
+                    for flow, head, eff in zip(points_x, points_head, points_eff)
+                ]
+                with multiprocessing.Pool() as pool:
+                    if flow_type == "volumetric":
+                        points += pool.map(create_points_flow_v, args_list)
+                    else:
+                        points += pool.map(create_points_flow_m, args_list)
+
+            if power_curves:
+                points_power = power_interpolated(points_x)
+                args_list = [
+                    (
+                        suc,
+                        Q_(float(speed), speed_units),
+                        Q_(flow, flow_units),
+                        Q_(head, head_units),
+                        Q_(power, power_units),
+                        b,
+                        D,
+                    )
+                    for flow, head, power in zip(points_x, points_head, points_power)
+                ]
+                with multiprocessing.Pool() as pool:
+                    if flow_type == "volumetric":
+                        points += pool.map(create_points_flow_v_power, args_list)
+                    else:
+                        points += pool.map(create_points_flow_m_power, args_list)
 
         return cls(points)
 
@@ -732,7 +753,11 @@ class Impeller:
         number_of_points=10,
         flow_units="m**3/s",
         head_units="J/kg",
+        power_units="W",
         speed_units="RPM",
+        head_interpolation_method="interp1d",
+        eff_interpolation_method="interp1d",
+        power_interpolation_method="interp1d",
     ):
         """Convert points from csv generated by engauge to csv with 6 points at same flow for use on hysys.
 
@@ -768,26 +793,56 @@ class Impeller:
         head_units : str
             Head units used when extracting data with engauge.
             If the curve head units are in meter you can use: head_units="m*g0".
+        power_units : str
+            Power units used when extracting data with engauge.
         speed_units : str
             Speed units used when extracting data with engauge.
+        head_interpolation_method : str, optional
+            Interpolation method from scipy. Can be interp1d or UnivariateSpline.
+            Default is interp1d.
+        eff_interpolation_method : str, optional
+            Interpolation method from scipy. Can be interp1d or UnivariateSpline.
+            Default is interp1d.
+        power_interpolation_method : str, optional
+            Interpolation method from scipy. Can be interp1d or UnivariateSpline.
+            Default is interp1d.
         """
         head_path = curve_path / (curve_name + "-head.csv")
         eff_path = curve_path / (curve_name + "-eff.csv")
+        power_path = curve_path / (curve_name + "-power.csv")
 
         head_curves = read_data_from_engauge_csv(head_path)
-        eff_curves = read_data_from_engauge_csv(eff_path)
-
-        return cls.load_from_dict(
-            suc=suc,
-            b=b,
-            D=D,
-            head_curves=head_curves,
-            eff_curves=eff_curves,
-            number_of_points=number_of_points,
-            flow_units=flow_units,
-            head_units=head_units,
-            speed_units=speed_units,
-        )
+        if eff_path.is_file():
+            eff_curves = read_data_from_engauge_csv(eff_path)
+            return cls.load_from_dict(
+                suc=suc,
+                b=b,
+                D=D,
+                head_curves=head_curves,
+                eff_curves=eff_curves,
+                number_of_points=number_of_points,
+                flow_units=flow_units,
+                head_units=head_units,
+                speed_units=speed_units,
+                head_interpolation_method=head_interpolation_method,
+                eff_interpolation_method=eff_interpolation_method,
+            )
+        if power_path.is_file():
+            power_curves = read_data_from_engauge_csv(power_path)
+            return cls.load_from_dict(
+                suc=suc,
+                b=b,
+                D=D,
+                head_curves=head_curves,
+                power_curves=power_curves,
+                number_of_points=number_of_points,
+                flow_units=flow_units,
+                head_units=head_units,
+                power_units=power_units,
+                speed_units=speed_units,
+                head_interpolation_method=head_interpolation_method,
+                power_interpolation_method=power_interpolation_method,
+            )
 
     def save(self, file):
         """Save impeller to a toml file.
@@ -966,6 +1021,34 @@ def create_points_flow_m(x):
     )
 
 
+def create_points_flow_v_power(x):
+    """Helper function used to parallelize creation of points."""
+    suc, speed, flow, head, power, b, D = x
+    return Point(
+        suc=suc,
+        speed=speed,
+        flow_v=flow,
+        head=head,
+        power=power,
+        b=b,
+        D=D,
+    )
+
+
+def create_points_flow_m_power(x):
+    """Helper function used to parallelize creation of points."""
+    suc, speed, flow, head, power, b, D = x
+    return Point(
+        suc=suc,
+        speed=speed,
+        flow_m=flow,
+        head=head,
+        power=power,
+        b=b,
+        D=D,
+    )
+
+
 def calc_min_head_point(x, speed, imp, min_head):
     """Helper function to calculate min_head point."""
     try:
