@@ -1,5 +1,6 @@
 import pytest
 import numpy as np
+import pickle
 from pathlib import Path
 from tempfile import tempdir
 from numpy.testing import assert_allclose
@@ -249,6 +250,199 @@ def test_impeller_point(imp3):
         assert "Expected point is being extrapolated" in record[0].message.args[0]
 
 
+def test_conversion(imp3):
+    new_suc = ccp.State.define(p=Q_(2000, "kPa"), T=300, fluid={"co2": 1})
+    new_imp3 = ccp.Impeller.convert_from(imp3, suc=new_suc)
+
+    # fmt: off
+    expected_data = (
+        {
+            'name': '610.0 radian / second',
+            'x': np.array([0.41912405, 0.43130594, 0.44348782, 0.45566971, 0.46785159, 0.48003348,
+                 0.49221536, 0.50439725, 0.51657914, 0.52876102, 0.54094291, 0.55312479,
+                 0.56530668, 0.57748857, 0.58967045, 0.60185234, 0.61403422, 0.62621611,
+                 0.63839799, 0.65057988, 0.66276177, 0.67494365, 0.68712554, 0.69930742,
+                 0.71148931, 0.7236712 , 0.73585308, 0.74803497, 0.76021685, 0.77239874]),
+            'y': np.array([39289.31488196, 39344.84790727, 39303.6880876 , 39175.43439992,
+                 38969.68582119, 38696.04132836, 38364.09989839, 37983.46050824,
+                 37563.72213487, 37114.48375523, 36645.34434629, 36165.90288501,
+                 35685.57453847, 35206.62886529, 34722.05302594, 34224.21382259,
+                 33705.4780574 , 33158.21253254, 32574.9964841 , 31951.58778887,
+                 31286.19124756, 30577.07460428, 29822.50560311, 29020.75198815,
+                 28170.08150349, 27268.76189323, 26315.06090146, 25307.24627228,
+                 24243.58574978, 23122.34707806])
+         },
+        {
+            'name': '684.0 radian / second',
+            'x': np.array([0.46966959, 0.48405431, 0.49843902, 0.51282374, 0.52720845, 0.54159317,
+                 0.55597788, 0.57036259, 0.58474731, 0.59913202, 0.61351674, 0.62790145,
+                 0.64228617, 0.65667088, 0.67105559, 0.68544031, 0.69982502, 0.71420974,
+                 0.72859445, 0.74297917, 0.75736388, 0.77174859, 0.78613331, 0.80051802,
+                 0.81490274, 0.82928745, 0.84367217, 0.85805688, 0.87244159, 0.88682631]),
+            'y': np.array([49262.68664941, 49372.55805869, 49344.95140393, 49193.30856685,
+                 48931.07142915, 48571.68187254, 48128.58177873, 47615.21302943,
+                 47045.01750635, 46431.4370912 , 45787.91366568, 45127.88911151,
+                 44464.55578681, 43801.4058204 , 43129.33040014, 42438.37857178,
+                 41718.5993811 , 40960.04187384, 40152.99105682, 39291.26261319,
+                 38371.39014774, 37389.97717965, 36343.62722808, 35228.94381223,
+                 34042.53045126, 32780.99066436, 31440.92797069, 30018.94588944,
+                 28511.64793979, 26915.6376409 ])
+        },
+        {
+            'name': '742.0 radian / second',
+            'x': np.array([0.5113496 , 0.52677044, 0.54219128, 0.55761212, 0.57303296, 0.5884538 ,
+                 0.60387464, 0.61929548, 0.63471632, 0.65013716, 0.66555801, 0.68097885,
+                 0.69639969, 0.71182053, 0.72724137, 0.74266221, 0.75808305, 0.77350389,
+                 0.78892473, 0.80434557, 0.81976641, 0.83518725, 0.85060809, 0.86602893,
+                 0.88144977, 0.89687061, 0.91229146, 0.9277123 , 0.94313314, 0.95855398]),
+            'y': np.array([57611.59859346, 57707.57083545, 57664.41602198, 57494.33376897,
+                 57209.52369233, 56822.18540799, 56344.51853186, 55788.72267986,
+                 55166.99746792, 54491.54251194, 53774.55742785, 53028.24183156,
+                 52264.58074003, 51487.21663537, 50688.95475185, 49861.87605224,
+                 48998.06149931, 48089.59205582, 47128.70498871, 46109.97633847,
+                 45029.78253812, 43884.54633303, 42670.69046855, 41384.63769004,
+                 40022.81074287, 38581.63237241, 37057.525324  , 35446.91234302,
+                 33746.21617482, 31951.85956477])
+        }
+    )
+    # fmt: on
+    fig = new_imp3.head_plot()
+    for exp_data, act_data in zip(expected_data, fig.data):
+        assert exp_data["name"] == act_data["name"]
+        assert_allclose(exp_data["x"], act_data["x"])
+        assert_allclose(exp_data["y"], act_data["y"])
+
+
+def test_impeller_from_head_power(imp3):
+    power_curves = {
+        "9300": {
+            "x": [
+                0.6687254013790117,
+                0.7814575427138895,
+                0.8941896840487671,
+                1.006921825383645,
+                1.1196539667185226,
+                1.2323861080534004,
+            ],
+            "y": [
+                2845922.6805825057,
+                3127033.256978862,
+                3277599.4482584326,
+                3346364.051940953,
+                3318245.9907083097,
+                3142309.4965246622,
+            ],
+        },
+        "10463": {
+            "x": [
+                0.7519674970181466,
+                0.8855457803584165,
+                1.0191240636986865,
+                1.1527023470389566,
+                1.2862806303792265,
+                1.4198589137194964,
+            ],
+            "y": [
+                4040323.419778452,
+                4470614.322645651,
+                4679924.34330235,
+                4766884.742457078,
+                4692192.145169183,
+                4371260.275382653,
+            ],
+        },
+        "11373": {
+            "x": [
+                0.8212055999448744,
+                0.9648438234594419,
+                1.108482046974009,
+                1.2521202704885768,
+                1.3957584940031438,
+                1.5393967175177115,
+            ],
+            "y": [
+                5187282.603054684,
+                5735996.254573646,
+                6014112.857258716,
+                6112922.466683052,
+                6014449.752171731,
+                5637883.547516629,
+            ],
+        },
+    }
+    head_curves = {
+        "9300": {
+            "x": [
+                0.6687254013790117,
+                0.7814575427138895,
+                0.8941896840487671,
+                1.006921825383645,
+                1.1196539667185226,
+                1.2323861080534004,
+            ],
+            "y": [
+                100019.55831252398,
+                97843.926474612,
+                91333.49180838456,
+                83829.5315869165,
+                73455.67167543288,
+                58863.0,
+            ],
+        },
+        "10463": {
+            "x": [
+                0.7519674970181466,
+                0.8855457803584165,
+                1.0191240636986865,
+                1.1527023470389566,
+                1.2862806303792265,
+                1.4198589137194964,
+            ],
+            "y": [
+                126279.06010603686,
+                123614.59621056826,
+                114658.89031020318,
+                104184.87825996456,
+                89711.9654863798,
+                68995.04786728641,
+            ],
+        },
+        "11373": {
+            "x": [
+                0.8212055999448744,
+                0.9648438234594419,
+                1.108482046974009,
+                1.2521202704885768,
+                1.3957584940031438,
+                1.5393967175177115,
+            ],
+            "y": [
+                148586.0,
+                145581.59248034787,
+                135586.9563114437,
+                123053.76981279982,
+                106048.53829138467,
+                82407.0,
+            ],
+        },
+    }
+
+    imp = Impeller.load_from_dict(
+        suc=imp3.points[0].suc,
+        b=imp3.points[0].b,
+        D=imp3.points[0].D,
+        head_curves=head_curves,
+        power_curves=power_curves,
+        number_of_points=6,
+        flow_units="m³/s",
+        head_units="J/kg",
+    )
+
+    assert_allclose(imp.head, imp3.head)
+    assert_allclose(imp.power, imp3.power)
+    assert_allclose(imp.eff, imp3.eff)
+
+
 def test_impeller_curve():
     imp = impeller_example()
     c0 = imp.curve(speed=900)
@@ -583,3 +777,155 @@ def test_load_from_dict_isis():
     assert_allclose(p0.eff, 0.782169, rtol=1e-4)
     assert_allclose(p0.head, 97729.49349, rtol=1e-4)
     assert_allclose(p0.power, 3130330.074989, rtol=1e-4)
+
+
+def test_pickle(imp0):
+    pickled_imp0 = pickle.loads(pickle.dumps(imp0))
+    assert pickled_imp0 == imp0
+    assert hasattr(imp0, "head_plot") is True
+    assert hasattr(pickled_imp0, "head_plot") is True
+
+
+def test_interpolation_warning():
+    eff_curve = {
+        1000: {
+            "x": [
+                9.1937,
+                9.5582,
+                10.4576,
+                11.4588,
+                12.7945,
+                13.9998,
+                14.8382,
+                15.3756,
+                15.4093,
+                15.712,
+                15.9811,
+                16.2507,
+                16.5203,
+                16.7565,
+                16.8578,
+                17.0605,
+                17.2634,
+            ],
+            "y": [
+                0.662945,
+                0.673879,
+                0.684813,
+                0.690723,
+                0.696042,
+                0.690427,
+                0.681562,
+                0.673288,
+                0.672401,
+                0.666491,
+                0.660876,
+                0.653784,
+                0.646396,
+                0.639303,
+                0.635757,
+                0.628665,
+                0.620982,
+            ],
+        }
+    }
+
+    head_curve = {
+        1000: {
+            "x": [
+                9.1623,
+                9.9946,
+                9.9948,
+                10.7943,
+                11.5938,
+                12.3933,
+                12.4933,
+                13.293,
+                14.0929,
+                14.8597,
+                14.993,
+                15.7599,
+                16.4936,
+                17.2278,
+            ],
+            "y": [
+                70.896,
+                70.513,
+                70.115,
+                68.537,
+                66.96,
+                65.383,
+                64.987,
+                63.011,
+                60.638,
+                57.867,
+                57.471,
+                54.302,
+                51.132,
+                46.768,
+            ],
+        }
+    }
+
+    fluid = dict(
+        n2=1,
+    )
+    suc = State.define(p=Q_(62.7, "bar"), T=Q_(31.2, "degC"), fluid=fluid)
+
+    with pytest.warns(UserWarning) as record:
+        ccp.Impeller.load_from_dict(
+            suc=suc, head_curves=head_curve, eff_curves=eff_curve, b=1, D=1
+        )
+        assert "Head interpolation error in speed 1000 RPM" in record[0].message.args[0]
+
+    # faster to load than impeller_example
+    composition_fd = dict(
+        n2=0.4,
+        co2=0.22,
+        methane=92.11,
+        ethane=4.94,
+        propane=1.71,
+        ibutane=0.24,
+        butane=0.3,
+        ipentane=0.04,
+        pentane=0.03,
+        hexane=0.01,
+    )
+    suc_fd = State.define(p=Q_(3876, "kPa"), T=Q_(11, "degC"), fluid=composition_fd)
+
+
+def test_univariate_spline():
+    test_dir = Path(__file__).parent
+    curve_path = test_dir / "data"
+    curve_name = "normal"
+
+    composition_fd = dict(
+        n2=0.4,
+        co2=0.22,
+        methane=92.11,
+        ethane=4.94,
+        propane=1.71,
+        ibutane=0.24,
+        butane=0.3,
+        ipentane=0.04,
+        pentane=0.03,
+        hexane=0.01,
+    )
+    suc_fd = State.define(p=Q_(3876, "kPa"), T=Q_(11, "degC"), fluid=composition_fd)
+    imp3 = Impeller.load_from_engauge_csv(
+        suc=suc_fd,
+        curve_name=curve_name,
+        curve_path=curve_path,
+        b=Q_(10.6, "mm"),
+        D=Q_(390, "mm"),
+        number_of_points=6,
+        flow_units="kg/h",
+        head_units="kJ/kg",
+        head_interpolation_method="UnivariateSpline",
+        eff_interpolation_method="UnivariateSpline",
+    )
+
+    p0 = imp3.point(flow_m=Q_(90184, "kg/h"), speed=Q_(9300, "RPM"))
+    assert_allclose(p0.eff, 0.782169, rtol=1e-2)
+    assert_allclose(p0.head, 97729.49349, rtol=1e-2)
+    assert_allclose(p0.power, 3130330.074989, rtol=1e-2)
